@@ -1,18 +1,21 @@
 import collections
 import datetime as dt
 from abc import ABC, abstractmethod
-from dataclasses import is_dataclass, replace
+from dataclasses import replace
 from enum import Enum
 from pathlib import Path
 from textwrap import dedent
-from typing import Dict, Optional, Tuple
+from typing import Any, Dict, List, Optional, Tuple, Union, cast
 
 import cf_xarray
 import cftime
 import numpy as np
 import xarray as xr
+from numpy.distutils.misc_util import is_sequence
 
 from ravenpy.config.commands import (
+    HRU,
+    BaseDataCommand,
     BasinIndexCommand,
     BasinStateVariablesCommand,
     ChannelProfileCommand,
@@ -25,9 +28,12 @@ from ravenpy.config.commands import (
     HRUStateVariableTableCommand,
     LandUseClassesCommand,
     ObservationDataCommand,
+    ReservoirCommand,
+    SBGroupPropertyMultiplierCommand,
     SoilClassesCommand,
     SoilProfilesCommand,
     StationForcingCommand,
+    Sub,
     SubBasinGroupCommand,
     SubBasinsCommand,
     VegetationClassesCommand,
@@ -38,37 +44,33 @@ class RV(ABC):
     def __init__(self, config, **kwds):
         # Each RV has a reference to their parent object in order to access sibling RVs.
         self._config = config
+
         self.is_ostrich_tmpl = False
+
+        # This variable contains the RV file content when it was set from a file; if still
+        # None at the moment Raven is called, it means the corresponding RV must be rendered
+        # with the `to_rv` method.
         self.content = None
 
+        # This contains extra attributes that might be used with a customized template
+        # (currently used with HBVEC and MOHYSE emulators, for values in their RVH)
+        self._extra_attributes = {}
+
     def update(self, key, value):
         if hasattr(self, key):
-            # Special case: the user might be trying to update `params` or `derived_params`,
-            # (which are dataclasses) by supplying a list of values (either a list, tuple of
-            # numpy array); if that's the case, cast those values into a new instance of the
-            # corresponding dataclass.
-            attr = getattr(self, key)
-            if is_dataclass(attr) and isinstance(value, (list, tuple, np.ndarray)):
-                value = attr.__class__(*value)
             setattr(self, key, value)
             return True
         return False
 
-    def get_extra_attributes(self, d):
-        """
-        Not sure about this: for the moment I use it only for certain params that must
-        be injected in the RVH by the MOHYSE emulator. The idea is to complete the `d`
-        dict used in the `to_rv` method for the template with extra attributes that have
-        been added in the emulator.
-        """
-        e = {}
-        for k, v in self.__dict__.items():
-            if k not in d:
-                e[k] = v
-        return e
+    def set_extra_attributes(self, **kwargs):
+        for k, v in kwargs.items():
+            self._extra_attributes[k] = v
+
+    def get_extra_attribute(self, k):
+        return self._extra_attributes[k]
 
     def set_tmpl(self, tmpl, is_ostrich=False):
-        self.tmpl = tmpl
+        self.tmpl = tmpl  # type: ignore
         self.is_ostrich_tmpl = is_ostrich
 
     @property
@@ -125,7 +127,7 @@ class RVC(RV):
             "basin_states": BasinStateVariablesCommand(self.basin_states),
         }
 
-        d.update(self.get_extra_attributes(d))
+        d.update(self._extra_attributes)
 
         return dedent(self.tmpl).format(**d)
 
@@ -155,13 +157,17 @@ class RVH(RV):
 
     def __init__(self, config):
         super().__init__(config)
-        self.hrus = ()
-        self.subbasins = ()
-        self.land_subbasin_ids = ()
-        self.land_subbasin_property_multiplier = None
-        self.lake_subbasin_ids = ()
-        self.lake_subbasin_property_multiplier = None
-        self.reservoirs = ()
+        self.hrus: Tuple[HRU, ...] = ()
+        self.subbasins: Tuple[Sub, ...] = ()
+        self.land_subbasin_ids: Tuple[int, ...] = ()
+        self.land_subbasin_property_multiplier: Optional[
+            SBGroupPropertyMultiplierCommand
+        ] = None
+        self.lake_subbasin_ids: Tuple[int, ...] = ()
+        self.lake_subbasin_property_multiplier: Optional[
+            SBGroupPropertyMultiplierCommand
+        ] = None
+        self.reservoirs: Tuple[ReservoirCommand, ...] = ()
 
     def to_rv(self):
         d = {
@@ -178,7 +184,7 @@ class RVH(RV):
             "reservoirs": "\n\n".join(map(str, self.reservoirs)),
         }
 
-        d.update(self.get_extra_attributes(d))
+        d.update(self._extra_attributes)
 
         return dedent(self.tmpl).format(**d)
 
@@ -281,7 +287,7 @@ class RVI(RV):
         super().__init__(config)
 
         # These are attributes that can be modified/set directly
-        self.run_name = "run"
+        self.run_name: Optional[str] = "run"
         self.run_index = 0
         self.raven_version = "3.X.X"
         self.time_step = 1.0
@@ -303,6 +309,19 @@ class RVI(RV):
         ]
         self._suppress_output = False
 
+    def configure_from_nc_data(self, fns):
+        with xr.open_mfdataset(fns, combine="by_coords") as ds:
+            start, end = ds.indexes["time"][0], ds.indexes["time"][-1]
+            cal = ds.time.encoding.get("calendar", "standard")
+
+        if self.start_date in [None, dt.datetime(1, 1, 1)]:
+            self.start_date = start
+
+        if self.end_date in [None, dt.datetime(1, 1, 1)]:
+            self.end_date = end
+
+        self.calendar = RVI.CalendarOptions(cal.upper())
+
     @property
     def start_date(self):
         return self._start_date
@@ -457,6 +476,8 @@ class RVI(RV):
         d["identifier"] = self._config.identifier
         d["now"] = dt.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
 
+        d.update(self._extra_attributes)
+
         t = dedent(self._pre_tmpl) + dedent(self.tmpl) + dedent(self._post_tmpl)
         return t.format(**d)
 
@@ -486,6 +507,24 @@ class RVP(RV):
         self.channel_profiles: Tuple[ChannelProfileCommand, ...] = ()
         self.avg_annual_runoff: Optional[float] = None
 
+    def update(self, key, value):
+        if key == "params":
+            if is_sequence(value):
+                self.params = self._config.model_cls.Params(*value)
+            else:
+                assert isinstance(value, self._config.model_cls.Params)
+                self.params = value
+            return True
+        elif key == "derived_params":
+            if is_sequence(value):
+                self.derived_params = self._config.model_cls.DerivedParams(*value)
+            else:
+                assert isinstance(value, self._config.model_cls.DerivedParams)
+                self.derived_params = value
+            return True
+        else:
+            return super().update(key, value)
+
     def to_rv(self):
         d = {
             "params": self.params,
@@ -499,6 +538,9 @@ class RVP(RV):
             if self.avg_annual_runoff
             else "",
         }
+
+        d.update(self._extra_attributes)
+
         return dedent(self.tmpl).format(**d)
 
 
@@ -546,7 +588,16 @@ class RVT(RV):
     def __init__(self, config):
         super().__init__(config)
 
-        self._var_cmds = {k: {} for k in RVT.NC_VARS.keys()}
+        # These are customized variable attributes specified by the user
+        self._var_specs: Dict[str, Dict[str, Any]] = {k: {} for k in RVT.NC_VARS.keys()}
+
+        # These are the actual variable as `commands.BaseDataCommand` objects
+        self._var_cmds: Dict[str, Optional[BaseDataCommand]] = {
+            k: None for k in RVT.NC_VARS.keys()
+        }
+
+        # Specifies whether the variables must be configured using file NC data
+        self._auto_nc_configure = True
 
         self.nc_index = 0
         self.grid_weights = None
@@ -555,14 +606,24 @@ class RVT(RV):
         self.monthly_ave_evaporation = None
         self.monthly_ave_temperature = None
 
-        self._nc_latitude = []
-        self._nc_longitude = []
-        self._nc_elevation = []
+        self._nc_latitude: Optional[xr.DataArray] = None
+        self._nc_longitude: Optional[xr.DataArray] = None
+        self._nc_elevation: Optional[xr.DataArray] = None
         self._number_grid_cells = 0
 
-    def add_nc_variable(self, **kwargs):
+    def _add_nc_variable(self, **kwargs):
         std_name = kwargs.get("name", kwargs["var_name_nc"])
+        # If the name is not the standard one, search for it
+        # TODO: reorganize NC_VARS so that the keys are the Raven names
+        if std_name not in RVT.NC_VARS:
+            for sn, rec in RVT.NC_VARS.items():
+                if rec["raven"] == kwargs["data_type"] or std_name in rec["alts"]:
+                    std_name = sn
+                    break
+            else:
+                assert False, f"{std_name} not found in the list of standard names"
         is_obs_var = kwargs.pop("is_observation", False)
+        cmd: BaseDataCommand
         if len(kwargs["dim_names_nc"]) == 1:
             if std_name == "water_volume_transport_in_river_channel" or is_obs_var:
                 cmd = ObservationDataCommand(**kwargs)
@@ -576,38 +637,41 @@ class RVT(RV):
         else:
             cmd = GriddedForcingCommand(**kwargs)
 
-        if isinstance(self._var_cmds.get(std_name, None), dict):
-            self._var_cmds[std_name] = replace(cmd, **self._var_cmds[std_name])
-        else:
-            self._var_cmds[std_name] = cmd
+        spec = self._var_specs[std_name]
+        self._var_cmds[std_name] = replace(cmd, **spec)
+
+    def set_nc_variables(self, nc_variables):
+        """
+        This is meant for manually setting the variables, and should prevent
+        automatic configuration from an nc file.
+        """
+        for nc_var in nc_variables:
+            self._add_nc_variable(**nc_var)
+        self._auto_nc_configure = False
 
     def configure_from_nc_data(self, fns):
 
-        # Important note: if the object at key `k` is a dict (as opposed to a `Command`),
-        # don't reset it because it contains initial user-defined config (for the future Command
-        # object at that particular key)
-        for std_name in RVT.NC_VARS:
-            v = self._var_cmds[std_name]
-            if not isinstance(v, dict):
-                self._var_cmds[std_name] = {}
+        assert self._auto_nc_configure is True
+
+        self._var_cmds = {k: None for k in RVT.NC_VARS.keys()}
 
         for fn in fns:
             with xr.open_dataset(fn) as ds:
                 try:
-                    self.nc_latitude = ds.cf["latitude"]
-                    self.nc_longitude = ds.cf["longitude"]
-                    self.nc_elevation = ds.cf["vertical"]
+                    self._nc_latitude = ds.cf["latitude"]
+                    self._nc_longitude = ds.cf["longitude"]
+                    self._nc_elevation = ds.cf["vertical"]
                 except KeyError:
                     # Will try to compute values later from first HRU (in self.to_rv)
                     pass
 
                 # Check if any alternate variable name is in the file.
                 for std_name in RVT.NC_VARS:
-                    for var_name in [std_name] + RVT.NC_VARS[std_name]["alts"]:
+                    for var_name in [std_name] + RVT.NC_VARS[std_name]["alts"]:  # type: ignore
                         if var_name not in ds.data_vars:
                             continue
                         nc_var = ds[var_name]
-                        self.add_nc_variable(
+                        self._add_nc_variable(
                             name=std_name,
                             file_name_nc=fn,
                             data_type=RVT.NC_VARS[std_name]["raven"],
@@ -619,8 +683,8 @@ class RVT(RV):
                         break
 
     def update(self, key, value):
-        if key in self._var_cmds:
-            self._var_cmds[key].update(value)
+        if key in self._var_specs:
+            self._var_specs[key].update(value)
             return True
         elif key == "nc_index":
             self.nc_index = value
@@ -642,25 +706,38 @@ class RVT(RV):
 
         use_gauge = any(type(cmd) is DataCommand for cmd in self._var_cmds.values())
         if use_gauge:
-            data = []
+            data_cmds = []
             for var, cmd in self._var_cmds.items():
                 if cmd and not isinstance(cmd, ObservationDataCommand):
-                    data.append(cmd)
-            lat = (
-                self._nc_latitude[self.nc_index]
-                if self._nc_latitude
-                else self._config.rvh.hrus[0].latitude
-            )
-            lon = (
-                self._nc_longitude[self.nc_index]
-                if self._nc_longitude
-                else self._config.rvh.hrus[0].longitude
-            )
-            elev = (
-                self._nc_elevation[self.nc_index]
-                if self._nc_elevation
-                else self._config.rvh.hrus[0].elevation
-            )
+                    cmd = cast(DataCommand, cmd)
+                    data_cmds.append(cmd)
+
+            if (
+                self._nc_latitude
+                and self._nc_latitude.shape
+                and len(self._nc_latitude) > self.nc_index
+            ):
+                lat = self._nc_latitude.values[self.nc_index]
+            else:
+                lat = self._config.rvh.hrus[0].latitude
+
+            if (
+                self._nc_longitude
+                and self._nc_longitude.shape
+                and len(self._nc_longitude) > self.nc_index
+            ):
+                lon = self._nc_longitude.values[self.nc_index]
+            else:
+                lon = self._config.rvh.hrus[0].longitude
+
+            if (
+                self._nc_elevation
+                and self._nc_elevation
+                and len(self._nc_elevation) > self.nc_index
+            ):
+                elev = self._nc_elevation.values[self.nc_index]
+            else:
+                elev = self._config.rvh.hrus[0].elevation
 
             d["gauge"] = GaugeCommand(
                 latitude=lat,
@@ -670,21 +747,22 @@ class RVT(RV):
                 snow_correction=self.snow_correction,
                 monthly_ave_evaporation=self.monthly_ave_evaporation,
                 monthly_ave_temperature=self.monthly_ave_temperature,
-                data=data,
-            )
+                data_cmds=tuple(data_cmds),
+            )  # type: ignore
         else:
             # Construct default grid weights applying equally to all HRUs
             data = [(hru.hru_id, self.nc_index, 1.0) for hru in self._config.rvh.hrus]
             gw = self.grid_weights or GridWeightsCommand(
                 number_hrus=len(data),
                 number_grid_cells=self._number_grid_cells,
-                data=data,
+                data=tuple(data),
             )
             cmds = []
             for var, cmd in self._var_cmds.items():
                 if cmd and not isinstance(cmd, ObservationDataCommand):
                     # TODO: implement a RedirectToFile mechanism to avoid inlining the grid weights
                     # multiple times as we do here
+                    cmd = cast(Union[GriddedForcingCommand, StationForcingCommand], cmd)
                     if len(cmd.grid_weights.data) == 1:
                         cmd.grid_weights = gw
                     cmds.append(cmd)
@@ -706,7 +784,7 @@ class RVT(RV):
                     )
                 # Set the :StationxIdx (which starts at 1)
                 cmd.index = self.nc_index + 1
-                d["observed_data"] = cmd
+                d["observed_data"] = cmd  # type: ignore
                 break
 
         return dedent(self.tmpl).format(**d)
@@ -733,6 +811,17 @@ class OST(RV):
         # If there's an OstRandomNumbers.txt file this is its path
         self.random_numbers_path = None
 
+    def update(self, key, value):
+        if key in ["lowerBounds", "upperBounds"]:
+            if is_sequence(value):
+                setattr(self, key, self._config.model_cls.Params(*value))
+            else:
+                assert isinstance(value, self._config.model_cls.Params)
+                setattr(self, key, value)
+            return True
+        else:
+            return super().update(key, value)
+
     @property
     def max_iterations(self):
         return self._max_iterations
@@ -783,7 +872,8 @@ class OST(RV):
 
 
 class Config:
-    def __init__(self, **kwargs):
+    def __init__(self, model_cls, **kwargs):
+        self.model_cls = model_cls
         self.rvc = RVC(self)
         self.rvh = RVH(self)
         self.rvi = RVI(self)
